# /usr/bin/env python3

from flask import Flask, jsonify, request
from flask_cors import CORS
import astral
import astral.sun
import datetime
import time
import argparse
import neopixel
import board
import RPi.GPIO as GPIO
import requests
import configparser

triggerOnTime = 0
lightStatus = False
useAstral = True

GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)

app = Flask(__name__)
CORS(app)


def verifyapikey(requestapikey):
    if requestapikey == apiKey:
        return True
    return False


# Update LED to reflect if trigger times by Astral are currently used
@app.route('/usingastral', methods=['GET'])
def usingastral():
    if verifyapikey(request.values.get('apikey')):
        global useAstral
        if useAstral is True:
            strip[0] = (0, 0, 255)
            time.sleep(0.2)
            if lightStatus is True:
                strip[0] = (0, 255, 0)
            else:
                strip[0] = (255, 0, 0)
            return jsonify(response='Astral is being used', currentstatus=useAstral), 200
        else:
            return jsonify(response='Astral is not being used', currentstatus=useAstral), 200
    return jsonify(response='invalid api key'), 401


# Update LED to reflect if the relay is open or closed (Used at startup)
@app.route('/lightstatus', methods=['POST'])
def lightstatus():
    if verifyapikey(request.values.get('apikey')):
        if lightStatus is False:
            strip[0] = (255, 0, 0)
        elif lightStatus is True:
            strip[0] = (0, 255, 0)
        return jsonify(response='Startup routine completed'), 200
    return jsonify(response='invalid api key'), 401


# Update LED to reflect if internet works
@app.route('/nointernet', methods=['POST'])
def nointernet():
    if verifyapikey(request.values.get('apikey')):
        strip[0] = (85, 85, 85)
        return jsonify(response='Internet not working'), 200
    return jsonify(response='invalid api key'), 401


# Override Switch for current status of Light
@app.route('/override', methods=['POST'])
def switchoverride():
    if verifyapikey(request.values.get('apikey')):
        global lightStatus
        # Turn on the light
        if lightStatus is False:
            # Turn on outside lights
            GPIO.setup(20, GPIO.OUT, initial=1)
            # Set the LED to green to indicate the lights outside are on
            strip[0] = (0, 255, 0)
            lightStatus = True
        # Turn off the light
        else:
            # Turn off outside lights
            GPIO.setup(20, GPIO.IN)
            # Set the LED to red to indicate the lights outside are off
            strip[0] = (255, 0, 0)
            lightStatus = False

        if int(centralLoggingEnabled):
            requests.post(baseUrl + '/raspapi/cmd',
                          data={'id': int(switchId), 'response': 'override button pressed, status: ' + str(lightStatus),
                                'apikey': srvApiKey})
            requests.post(baseUrl + '/raspapi/status',
                          data={'id': int(switchId), 'key': 'switch_status', 'value': int(lightStatus),
                                'apikey': srvApiKey})
        return jsonify(response='Toggled Lights', currentstatus=lightStatus), 200
    return jsonify(response='invalid api key'), 401


# Toggle Switch for current status of Light
@app.route('/toggleastral', methods=['POST'])
def toggleastral():
    if verifyapikey(request.values.get('apikey')):
        global useAstral
        useAstral = int(request.values.get('command'))

        if int(centralLoggingEnabled):
            requests.post(baseUrl + '/raspapi/cmd',
                          data={'id': int(switchId), 'response': 'toggled astral ' + str(useAstral), 'apikey': srvApiKey})
            requests.post(baseUrl + '/raspapi/status',
                          data={'id': int(switchId), 'key': 'astral_status', 'value': int(useAstral), 'apikey': srvApiKey})

        return jsonify(response='Toggled Astral', currentstatus=useAstral), 200
    return jsonify(response='invalid api key'), 401


@app.route('/switchon', methods=['POST'])
def switchon():
    if verifyapikey(request.values.get('apikey')):
        # Turn on the light
        GPIO.setup(20, GPIO.OUT, initial=1)
        # Set the LED to green to indicate the lights are on
        strip[0] = (0, 255, 0)
        global lightStatus
        lightStatus = True

        if int(centralLoggingEnabled):
            requests.post(baseUrl + '/raspapi/cmd',
                          data={'id': switchId, 'response': 'Turned switch' + str(lightStatus), 'apikey': srvApiKey})
            requests.post(baseUrl + '/raspapi/status',
                          data={'id': int(switchId), 'key': 'switch_status', 'value': int(lightStatus),
                                'apikey': srvApiKey})

        return jsonify(response='Switched the light on', currenstatus=lightStatus), 200
    return jsonify(response='invalid api key'), 401


@app.route('/switchoff', methods=['POST'])
def switchoff():
    if verifyapikey(request.values.get('apikey')):
        # Turn off the light
        GPIO.setup(20, GPIO.IN)
        # Set the LED color to red to indicate the lights are off
        strip[0] = (255, 0, 0)
        global lightStatus
        lightStatus = False

        if int(centralLoggingEnabled):
            requests.post(baseUrl + '/raspapi/cmd',
                          data={'id': int(switchId), 'response': 'Turned switch' + str(lightStatus), 'apikey': srvApiKey})
            requests.post(baseUrl + '/raspapi/status',
                          data={'id': int(switchId), 'key': 'switch_status', 'value': int(lightStatus),
                                'apikey': srvApiKey})

        return jsonify(response='Switched the light off', currentstatus=lightStatus), 200
    return jsonify(response='invalid api key'), 401


@app.route('/updatelocation', methods=['POST'])
def updatelocation():
    if verifyapikey(request.values.get('apikey')):
        config = configparser.ConfigParser()
        config.read('config.ini')
        config['location']['name'] = request.values.get('loc_name')
        config['location']['region'] = request.values.get('loc_region')
        config['location']['lat'] = request.values.get('loc_lat')
        config['location']['lon'] = request.values.get('loc_lon')
        with open('config.ini', 'w') as configfile:
            config.write(configfile)

        if int(centralLoggingEnabled):
            requests.post(baseUrl + '/raspapi/updatelocation',
                          data={'id': int(switchId),
                                'loc_name': request.values.get('loc_name'),
                                'loc_region': request.values.get('loc_region'),
                                'loc_lat': request.values.get('loc_lat'), 'loc_lon': request.values.get('loc_lon'),
                                'apikey': srvApiKey})
        return jsonify(response='success'), 200
    return jsonify(response='invalid api key'), 401


@app.route('/gettime', methods=['GET'])
def gettime():
    if verifyapikey(request.values.get('apikey')):
        config = configparser.ConfigParser()
        config.read('config.ini')
        name = config['location']['name']
        region = config['location']['region']
        lat = float(config['location']['lat'])
        lon = float(config['location']['lon'])
        timezone = config['location']['timezone']

        # Get Sunset time for location
        l = astral.LocationInfo(name=name, region=region, latitude=lat, longitude=lon, timezone=timezone)
        global triggerOnTime
        triggerOnTime = astral.sun.sunset(observer=l.observer,
                                          date=datetime.date(int(request.args.get('year')),
                                                             int(request.args.get('month')),
                                                             int(request.args.get('day'))))
        return jsonify(response=triggerOnTime), 200
    return jsonify(response='invalid api key'), 401


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', '--clear', action='store_true', help='clear the display on exit')
    args = parser.parse_args()
    strip = neopixel.NeoPixel(board.D18, 1, brightness=0.2)
    config = configparser.ConfigParser()
    config.read('config.ini')
    baseUrl = config['setup']['baseurl']
    switchId = config['setup']['switchid']
    apiKey = config['setup']['apikey']
    srvApiKey = config['setup']['srvapikey']
    centralLoggingEnabled = config['setup']['storedb']

    app.run(host='0.0.0.0')
